/* SPDX-License-Identifier: 0BSD */

/*
 * Author: Lasse Collin
 */

#define MY_TYPE VFT_APP
#define MY_NAME "xzdec"
#define MY_SUFFIX ".exe"
#define MY_DESC "xzdec decompression tool for .xz files"
#include "common_w32res.rc"
